#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "create.h"

#include <string.h>

gchar save_filename[1024] = { '\0' };
gchar save_fontname[1024] = { '\0' };
struct {
	unsigned int charwidth, charheight, cols, rows, firstcell, firstchar, lastchar;
} save_info;

static gboolean create_bmpfont_onpixmap_getdata(GdkPixmap *pixmap) {
	return create_bmpfont_onpixmap(pixmap,save_fontname,save_info.charwidth,save_info.charheight,save_info.cols,save_info.rows,save_info.firstcell,save_info.firstchar,save_info.lastchar);
}

static GdkPixmap *create_bmpfont_aspixmap_getdata(void) {
	return create_bmpfont_aspixmap(save_fontname,save_info.charwidth,save_info.charheight,save_info.cols,save_info.rows,save_info.firstcell,save_info.firstchar,save_info.lastchar);
}

static gboolean create_bmpfont_asfile_getdata(void) {
	return create_bmpfont_asfile(save_filename,save_fontname,save_info.charwidth,save_info.charheight,save_info.cols,save_info.rows,save_info.firstcell,save_info.firstchar,save_info.lastchar);
}

static unsigned int spin_button_get_uivalue_fromname(GtkWidget *from, const gchar *name) {
	GtkSpinButton *spin = GTK_SPIN_BUTTON(lookup_widget(from,name));

	g_assert(spin);

	return (unsigned int) gtk_spin_button_get_value(spin);
}

static void save_info_fill(GtkWidget *from) {
	save_info.charwidth  = spin_button_get_uivalue_fromname(from,"spinbutton_charwidth");
	save_info.charheight = spin_button_get_uivalue_fromname(from,"spinbutton_charheight");

	save_info.cols       = spin_button_get_uivalue_fromname(from,"spinbutton_cols");
	save_info.rows       = spin_button_get_uivalue_fromname(from,"spinbutton_rows");

	save_info.firstcell  = spin_button_get_uivalue_fromname(from,"spinbutton_firstcell");

	save_info.firstchar  = spin_button_get_uivalue_fromname(from,"spinbutton_firstchar");
	save_info.lastchar   = spin_button_get_uivalue_fromname(from,"spinbutton_lastchar");

#if 0
	g_print("cw:\t%u\nch:\t%u\ncols:\t%u\nrows:\t%u\nfcell:\t%u\nfchar:\t%u\nlchar:\t%u\n",
		save_info.charwidth,save_info.charheight,
		save_info.cols,save_info.rows,
		save_info.firstcell,save_info.firstchar,save_info.lastchar);
#endif
}

static void save_fontname_fill(GtkWidget *from) {
	GtkFontSelection *fontselection_main = GTK_FONT_SELECTION(lookup_widget(from,"fontselection_main"));

	strncpy(save_fontname,gtk_font_selection_get_font_name(fontselection_main),sizeof(save_fontname));

#if 0
	g_print("save_fontname: %s\n",save_fontname);
#endif
}

void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *fileselection_save = create_fileselection_save();

	if(save_filename[0] != '\0')
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection_save),save_filename);

	save_info_fill(GTK_WIDGET(menuitem));
	save_fontname_fill(GTK_WIDGET(menuitem));
	gtk_widget_show(fileselection_save);
}

void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(save_filename[0] == '\0')
		on_save_as1_activate(menuitem,user_data);
	else {
		save_info_fill(GTK_WIDGET(menuitem));
		save_fontname_fill(GTK_WIDGET(menuitem));
		create_bmpfont_asfile_getdata();
	}
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}

void
on_preview1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *dialog_preview = create_dialog_preview();

	save_info_fill(GTK_WIDGET(menuitem));
	save_fontname_fill(GTK_WIDGET(menuitem));

	g_assert(dialog_preview);

	{
		GtkWidget *scrolledwindow_preview = lookup_widget(dialog_preview,"scrolledwindow_preview");

		g_assert(scrolledwindow_preview);

		{
			GdkPixmap *pixmap = create_bmpfont_aspixmap_getdata();

			g_assert(pixmap);

			{
				GtkWidget *image = gtk_image_new_from_pixmap(pixmap,NULL);
	
				g_assert(image);
				
				gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow_preview),image);

				gtk_widget_show(image);
			}
		}
	}

	gtk_widget_show(dialog_preview);
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *dialog_about = create_dialog_about();

	gtk_widget_show(dialog_about);
}


void
on_window_main_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_button_save_ok_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fileselection_save = lookup_widget(GTK_WIDGET(button),"fileselection_save");

	g_assert(fileselection_save);

	{
		strncpy(save_filename,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection_save)),sizeof(save_filename));
	}

	create_bmpfont_asfile_getdata();

	gtk_widget_destroy(fileselection_save);
}


void
on_button_save_cancel_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fileselection_save = lookup_widget(GTK_WIDGET(button),"fileselection_save");

	g_assert(fileselection_save);

	gtk_widget_destroy(fileselection_save);
}


void
on_button_about_close_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *dialog_about = lookup_widget(GTK_WIDGET(button),"dialog_about");

	g_assert(dialog_about);

	gtk_widget_destroy(dialog_about);
}


void
on_button_preview_close_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *dialog_preview = lookup_widget(GTK_WIDGET(button),"dialog_preview");

	g_assert(dialog_preview);

	gtk_widget_destroy(dialog_preview);
}
