#include "create.h"

/*
U-00000000 - U-0000007F: 0xxxxxxx
U-00000080 - U-000007FF: 110xxxxx 10xxxxxx
U-00000800 - U-0000FFFF: 1110xxxx 10xxxxxx 10xxxxxx
U-00010000 - U-001FFFFF: 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
U-00200000 - U-03FFFFFF: 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
U-04000000 - U-7FFFFFFF: 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
*/

static unsigned int ucs4int_to_utf8str(unsigned int Num, gchar str[6]) {
	if(Num <= 0x0000007F) {
		str[0] = Num;
		return 1;
	} else if(Num <= 0x000007FF) {
		str[0] = 0xC0 | ((Num & 0x000007C0) >> 6);
		str[1] = 0x80 | (Num & 0x0000003F);
		return 2;
	} else if(Num <= 0x0000FFFF) {
		str[0] = 0xE0 | ((Num & 0x00007000) >> 6*2);
		str[1] = 0x80 | ((Num & 0x00000FC0) >> 6);
		str[2] = 0x80 | (Num & 0x0000003F);
		return 3;
	} else if(Num <= 0x001FFFFF) {
		str[0] = 0xF0 | ((Num & 0x001C0000) >> 6*3);
		str[1] = 0x80 | ((Num & 0x0003F000) >> 6*2);
		str[2] = 0x80 | ((Num & 0x00000FC0) >> 6);
		str[3] = 0x80 | (Num & 0x0000003F);
		return 4;
	} else if(Num <= 0x03FFFFFF) {
		str[0] = 0xF8 | ((Num & 0x03000000) >> 6*4);
		str[1] = 0x80 | ((Num & 0x00FC0000) >> 6*3);
		str[2] = 0x80 | ((Num & 0x0003F000) >> 6*2);
		str[3] = 0x80 | ((Num & 0x00000FC0) >> 6);
		str[4] = 0x80 | (Num & 0x0000003F);
		return 5;
	} else if(Num <= 0x7FFFFFFF) {
		str[0] = 0xFC | ((Num & 0x40000000) >> 6*5);
		str[1] = 0x80 | ((Num & 0x3F000000) >> 6*4);
		str[2] = 0x80 | ((Num & 0x00FC0000) >> 6*3);
		str[3] = 0x80 | ((Num & 0x0003F000) >> 6*2);
		str[4] = 0x80 | ((Num & 0x00000FC0) >> 6);
		str[5] = 0x80 | (Num & 0x0000003F);
		return 6;
	}
}

gboolean create_bmpfont_char_onpixmap(GdkPixmap *pixmap, GdkGC *gc, PangoContext *context, PangoFontDescription *description, const gchar *text, unsigned int textlen, unsigned int X, unsigned int Y, unsigned int charwidth, unsigned int charheight) {
	PangoLayout *layout = pango_layout_new(context);

	if(layout == NULL)
		return FALSE;

	pango_layout_set_font_description(layout,description);
	pango_layout_set_width(layout,charwidth);
	pango_layout_set_alignment(layout,PANGO_ALIGN_CENTER);
	pango_layout_set_text(layout,text,textlen);

	gdk_draw_layout(GDK_DRAWABLE(pixmap),gc,X,Y,layout);

	g_object_unref(G_OBJECT(layout));

	return TRUE;
}

gboolean create_bmpfont_onpixmap(GdkPixmap *pixmap, const gchar *fontname, unsigned int charwidth, unsigned int charheight, unsigned int cols, unsigned int rows, unsigned int firstcell, unsigned int firstchar, unsigned int lastchar) {
	unsigned int imagewidth  = charwidth*cols;
	unsigned int imageheight = charheight*rows;
	PangoContext *context = gdk_pango_context_get();

	if(context == NULL)
		return FALSE;

	{
		PangoFontDescription *description = pango_font_description_from_string(fontname);

		if(description == NULL) {
			g_object_unref(G_OBJECT(context));
			return FALSE;
		}

		{
			GdkGC *gc = gdk_gc_new(GDK_DRAWABLE(pixmap));

			if(gc == NULL) {
				g_object_unref(G_OBJECT(context));
				return FALSE;
			}
			
			{
				GdkColor fg, bg;

				fg.red = fg.green = fg.blue = 0x0000;
				bg.red = bg.green = bg.blue = 0x0000;

				gdk_gc_set_rgb_fg_color(gc,&fg);
				gdk_gc_set_rgb_bg_color(gc,&bg);
			}
			{
				gdk_draw_rectangle(GDK_DRAWABLE(pixmap),gc,TRUE,0,0,imagewidth,imageheight);
			}
			{
				GdkColor fg, bg;

				fg.red = fg.green = fg.blue = 0xFFFF;
				bg.red = bg.green = bg.blue = 0xFFFF;

				gdk_gc_set_rgb_fg_color(gc,&fg);
				gdk_gc_set_rgb_bg_color(gc,&bg);
			}
			{
				int align = 0;
				unsigned int curcell, curchar, maxcell = rows*cols;
				gchar text[6];

				for(curchar=firstchar, curcell=firstcell; curchar<=lastchar; ++curchar, ++curcell) {
					unsigned int c = curcell / cols;
					unsigned int r = curcell - (c * cols);

					unsigned int Y = (c * charheight);	
					unsigned int X = (r * charwidth) + (charwidth/2); // CENTERED
					
					if(curcell >= maxcell) {
						break;
					} else {
						unsigned int length = ucs4int_to_utf8str(curchar,text);

						create_bmpfont_char_onpixmap(pixmap,gc,context,description,text,length,
							X, Y,
							charwidth,charheight);
					}
				}
			}

			g_object_unref(G_OBJECT(gc));
		}

		pango_font_description_free(description);
	}

	g_object_unref(G_OBJECT(context));

	return TRUE;
}

GdkPixmap *create_bmpfont_aspixmap(const gchar *fontname, unsigned int charwidth, unsigned int charheight, unsigned int cols, unsigned int rows, unsigned int firstcell, unsigned int firstchar, unsigned int lastchar) {
	unsigned int imagewidth  = charwidth*cols;
	unsigned int imageheight = charheight*rows;
	GdkPixmap *pixmap = gdk_pixmap_new(NULL,imagewidth,imageheight,24);

	if(pixmap == NULL)
		return NULL;

	if(!create_bmpfont_onpixmap(pixmap,fontname,charwidth,charheight,cols,rows,firstcell,firstchar,lastchar)) {
		g_object_unref(G_OBJECT(pixmap));
		return NULL;
	}

	return pixmap;
}

static const char *guess_imagetype_from_filename(const char *filename) {
	static const char default_imagetype[] = "png";
	const char *extension = filename;

	while(*extension != '\0') ++extension;
	while(*extension != '.' && extension > filename) --extension;

	if(*(extension++) == '.') {
		static const char jpeg_imagetype[] = "jpeg";

		if(strcmp(extension,"jpg") == 0)
			return jpeg_imagetype;

		/* Hope it's something valid... */
		return extension;
	} else
		return default_imagetype;
}

gboolean create_bmpfont_asfile(const gchar *filename, const gchar *fontname, unsigned int charwidth, unsigned int charheight, unsigned int cols, unsigned int rows, unsigned int firstcell, unsigned int firstchar, unsigned int lastchar) {
	unsigned int imagewidth  = charwidth*cols;
	unsigned int imageheight = charheight*rows;
	GdkPixmap *pixmap = create_bmpfont_aspixmap(fontname,charwidth,charheight,cols,rows,firstcell,firstchar,lastchar);

	if(pixmap == NULL)
		return FALSE;

	{
		GdkPixbuf *pixbuf = gdk_pixbuf_get_from_drawable(NULL,GDK_DRAWABLE(pixmap),gdk_drawable_get_colormap(GDK_DRAWABLE(pixmap)),0,0,0,0,imagewidth,imageheight);
		GError *error;

		if(pixbuf == NULL) {
			g_object_unref(G_OBJECT(pixmap));
			return FALSE;
		}
		
		if(!gdk_pixbuf_save(pixbuf,filename,guess_imagetype_from_filename(filename),&error,NULL)) {
			g_object_unref(G_OBJECT(pixmap));
			g_object_unref(G_OBJECT(pixbuf));
			return FALSE;
		}

		g_object_unref(G_OBJECT(pixbuf));
	}

	g_object_unref(G_OBJECT(pixmap));

	return TRUE;
}
